<?php
/**
* 2007-2019 PrestaShop
*
* NOTICE OF LICENSE
*
* This source file is subject to the Academic Free License (AFL 3.0)
* that is bundled with this package in the file LICENSE.txt.
* It is also available through the world-wide-web at this URL:
* http://opensource.org/licenses/afl-3.0.php
* If you did not receive a copy of the license and are unable to
* obtain it through the world-wide-web, please send an email
* to license@prestashop.com so we can send you a copy immediately.
*
* DISCLAIMER
*
* Do not edit or add to this file if you wish to upgrade PrestaShop to newer
* versions in the future. If you wish to customize PrestaShop for your
* needs please refer to http://www.prestashop.com for more information.
*
*  @author    PrestaShop SA <contact@prestashop.com>
*  @copyright 2007-2019 PrestaShop SA
*  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
*  International Registered Trademark & Property of PrestaShop SA
*/

if (!defined('_PS_VERSION_')) {
    exit;
}

class Leopopupsale extends Module
{
    protected $config_form = false;

    public function __construct()
    {
        $this->name = 'leopopupsale';
        $this->tab = 'administration';
        $this->version = '1.0.0';
        $this->author = 'Leotheme';
        $this->need_instance = 0;
        $this->bootstrap = true;
        parent::__construct();
        $this->displayName = $this->l('Leo Popup Sale');
        $this->description = $this->l('This is great module for create sale popup');
        $this->ps_versions_compliancy = array('min' => '1.6', 'max' => _PS_VERSION_);
        $this->context = Context::getContext();
        $this->id_lang = $this->context->language->id;
    }

    public function install()
    {
        Configuration::updateValue('LEOPOPUPSALE_ACTIVE', '1');

        return parent::install() &&
            $this->registerHook('header') &&
            $this->registerHook('ActionAdminControllerSetMedia') &&
            $this->registerHook('displayFooter') &&
            $this->registerHook('displayFooterAfter');
    }

    public function uninstall()
    {
        Configuration::deleteByName('LEOPOPUPSALE_ACTIVE');

        return parent::uninstall();
    }

    public function getContent()
    {
        $output = '';
        $errors = array();

        if (((bool)Tools::isSubmit('submitLeopopupsaleModule')) == true) {
            $time = Tools::getValue('LEOPOPUPSALE_TIME');

            if (!Validate::isInt($time)) {
                $errors[] = $this->trans(
                    'Invalid value for the Time popup',
                    array(),
                    'Modules.Leopopupsale.Admin'
                );
            }

            if (isset($errors) && count($errors)) {
                $output = $this->displayError(implode('<br />', $errors));
            } else {
                $this->postProcess();

                $output = $this->displayConfirmation(
                    $this->trans(
                        'The settings have been updated.',
                        array(),
                        'Admin.Notifications.Success'
                    )
                );
            }
        }

        $this->context->smarty->assign('module_dir', $this->_path);

        return $output.$this->renderForm();
    }

    protected function renderForm()
    {
        $helper = new HelperForm();

        $helper->show_toolbar = false;
        $helper->table = $this->table;
        $helper->module = $this;
        $helper->default_form_language = $this->context->language->id;
        $helper->allow_employee_form_lang = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG', 0);

        $helper->identifier = $this->identifier;
        $helper->submit_action = 'submitLeopopupsaleModule';
        $helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false)
            .'&configure='.$this->name.'&tab_module='.$this->tab.'&module_name='.$this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');

        $helper->tpl_vars = array(
            'fields_value' => $this->getConfigFormValues(), /* Add values for your inputs */
            'languages' => $this->context->controller->getLanguages(),
            'id_language' => $this->context->language->id,
        );

        return $helper->generateForm(array($this->getConfigForm()));
    }

    public function getProduct($id_product = '')
    {
        if ($id_product != '') {
            return Db::getInstance()->executeS(
                'SELECT p.`id_product`, pl.`name`, pl.`link_rewrite`
                FROM `'._DB_PREFIX_.'product` p
                LEFT JOIN `'._DB_PREFIX_.'product_lang` pl
                ON p.`id_product` = pl.`id_product`
                WHERE p.`active` = 1 AND p.`id_product` IN ('.pSQL($id_product).')
                AND pl.`id_lang` = '.(int)$this->id_lang
            );
        } else {
            return array();
        }
    }

    protected function getConfigForm()
    {
        $categories = $this->getCategory();

        if (!empty($categories)) {
            foreach ($categories as $category) {
                $query[] = array(
                    'id_option' => $category['id_category'], 
                    'name' => $category['name'],
                );
            }
        }

        $query1 = array(
            array(
                'id_option' => 1,
                'name' => $this->l('Top-Left')
            ),
            array(
                'id_option' => 2,
                'name' => $this->l('Top-Right')
            ),
            array(
                'id_option' => 3,
                'name' => $this->l('Bottom-Left')
            ),
            array(
                'id_option' => 4,
                'name' => $this->l('Bottom-Right')
            ),
        );

        $this->smarty->assign(array(
            'products' => $this->getProduct(Configuration::get('LEOPOPUPSALE_LIST_PRODUCT')),
            'customers' => unserialize($this->base64Decode(
                Configuration::get('LEOPOPUPSALE_CUSTOMER')
            ))
        ));

        $html = $this->fetch('module:leopopupsale/views/templates/leo-form.tpl');

        return array(
            'form' => array(
                'legend' => array(
                'title' => $this->l('Settings'),
                'icon' => 'icon-cogs',
                ),
                'input' => array(
                    array(
                        'type' => 'html',
                        'name' => 'html_data1',
                        'html_content' =>  '<div class="alert alert-danger table-customer"><p>'.
                        $this->l('This is version free. Clik to down load version Pro').'</p><a href="https://www.leotheme.com/module/prestashop-modules-membership.html">https://www.leotheme.com/module/prestashop-modules-membership.html</a></div>',
                    ),
                    array(
                        'type' => 'switch',
                        'label' => $this->l('Active'),
                        'name' => 'LEOPOPUPSALE_ACTIVE',
                        'is_bool' => true,
                        'desc' => $this->l('Use this module in live mode'),
                        'values' => array(
                            array(
                                'id' => 'active_on',
                                'value' => 1,
                                'label' => $this->l('Enabled')
                            ),
                            array(
                                'id' => 'active_off',
                                'value' => 0,
                                'label' => $this->l('Disabled')
                            )
                        ),
                    ),
                    array(
                        'type' => 'switch',
                        'label' => $this->l('Config front end'),
                        'name' => 'LEOPOPUPSALE_CONFIG_ACTIVE',
                        'is_bool' => true,
                        'values' => array(
                            array(
                                'id' => 'active_on',
                                'value' => 1,
                                'label' => $this->l('Enabled')
                            ),
                            array(
                                'id' => 'active_off',
                                'value' => 0,
                                'label' => $this->l('Disabled')
                            )
                        ),
                    ),
                    array(
                        'type' => 'select',
                        'label' => $this->l('Select THEME'),
                        'name' => 'LEOPOPUPSALE_SELECT_THEME',
                        'options' => array(
                            'query' => array(
                                array(
                                    'id_option' => 'basic',
                                    'name' => $this->l('Basic'),
                                ),
                                array(
                                    'id_option' => 'model',
                                    'name' => $this->l('Model'),
                                ),
                                array(
                                    'id_option' => 'smart',
                                    'name' => $this->l('Smart'),
                                ),
                            ),
                            'id' => 'id_option', 
                            'name' => 'name'
                        ),
                    ),
                    array(
                        'col' => 3,
                        'type' => 'text',
                        'name' => 'LEOPOPUPSALE_TIME',
                        'label' => $this->l('Time popup'),
                        'desc' => $this->l('Default 1000'),
                    ),
                    array(
                        'type' => 'color',
                        'name' => 'LEOPOPUPSALE_BG',
                        'label' => $this->l('Color Background'),
                    ),
                    array(
                        'type' => 'select',
                        'label' => $this->l('Select Position'),
                        'name' => 'LEOPOPUPSALE_POSITION',
                        'options' => array(
                            'query' => $query1,
                            'id' => 'id_option', 
                            'name' => 'name'
                        ),
                    ),
                    array(
                        'type' => 'select',
                        'label' => $this->l('Select Type'),
                        'name' => 'LEOPOPUPSALE_SELECT_TYPE',
                        'options' => array(
                            'query' => array(
                                array(
                                    'id_option' => 1,
                                    'name' => $this->l('Categories'),
                                ),
                                array(
                                    'id_option' => 2,
                                    'name' => $this->l('Orders'),
                                ),
                                array(
                                    'id_option' => 3,
                                    'name' => $this->l('Custom'),
                                ),
                            ),
                            'id' => 'id_option', 
                            'name' => 'name'
                        ),
                    ),
                    array(
                        'type' => 'select',
                        'label' => $this->l('Select Category'),
                        'name' => 'LEOPOPUPSALE_CATEGORY',
                        'options' => array(
                            'query' => $query,
                            'id' => 'id_option', 
                            'name' => 'name'
                        ),
                        'desc' => $this->l('Product in category selected will display'),
                    ),
                    array(
                        'type' => 'html',
                        'name' => 'html_data',
                        'html_content' =>  $html,
                    ),
                ),
                'submit' => array(
                    'title' => $this->l('Save'),
                ),
            ),
        );
    }

    protected function getConfigFormValues()
    {
        return array(
            'LEOPOPUPSALE_ACTIVE' => Configuration::get('LEOPOPUPSALE_ACTIVE'),
            'LEOPOPUPSALE_CONFIG_ACTIVE' => Configuration::get('LEOPOPUPSALE_CONFIG_ACTIVE'),
            'LEOPOPUPSALE_SELECT_THEME' => Configuration::get('LEOPOPUPSALE_SELECT_THEME'),
            'LEOPOPUPSALE_TIME' => Configuration::get('LEOPOPUPSALE_TIME', '1000'),
            'LEOPOPUPSALE_CATEGORY' => Configuration::get('LEOPOPUPSALE_CATEGORY'),
            'LEOPOPUPSALE_POSITION' => Configuration::get('LEOPOPUPSALE_POSITION'),
            'LEOPOPUPSALE_SELECT_TYPE' => Configuration::get('LEOPOPUPSALE_SELECT_TYPE'),
            'LEOPOPUPSALE_BG' => Configuration::get('LEOPOPUPSALE_BG'),
        );
    }

    protected function postProcess()
    {
        $form_values = $this->getConfigFormValues();

        foreach (array_keys($form_values) as $key) {
            Configuration::updateValue($key, Tools::getValue($key));
        }

        if (Tools::getValue('LEOPOPUPSALE_SELECT_TYPE') == 3) {
            Configuration::updateValue('LEOPOPUPSALE_LIST_PRODUCT', Tools::getValue('LEOPOPUPSALE_LIST_PRODUCT'));

            $customer = array();
            $name = Tools::getValue('leo_name');
            $phone = Tools::getValue('leo_phone');

            if (!empty($name)) {
                $count = 1;

                for ($i=0; $i < count($name); $i++) { 
                    $customer[$i]['name'] = $name[$i];
                    $customer[$i]['phone'] = $phone[$i];
                }
            }

            Configuration::updateValue(
                'LEOPOPUPSALE_CUSTOMER',
                $this->base64Encode(serialize($customer))
            );
        }
    }

    public function hookHeader()
    {
        $this->context->controller->addJS($this->_path.'/views/js/cookie.js');
        $this->context->controller->addJS($this->_path.'/views/js/front.js');

        if (Configuration::get('LEOPOPUPSALE_CONFIG_ACTIVE') != 0) {
            $this->context->controller->addJqueryPlugin('colorpicker');

            Media::addJsDef(array(
               'baseAdminDir' => __PS_BASE_URI__.'/', 
            ));
        }

        if (Tools::getIsset('submit_config')) {
            $cookie = array(
                'time' => Tools::getValue('time'),
                'color' => Tools::getValue('color'),
                'position' => Tools::getValue('position'),
            );

            $this->context->cookie->__set('cookie_popup', Tools::jsonEncode($cookie));
        }

        $this->context->controller->addCSS($this->_path.'/views/css/front.css');
    }

    public function hookActionAdminControllerSetMedia()
    {
        $this->context->controller->addJS($this->_path.'/views/js/back.js');
        $this->context->controller->addCSS($this->_path.'/views/css/back.css');

        Media::addJsDef(array(
            'url' => $this->context->shop->getBaseURL(true, true) . 'modules/leopopupsale/ajax.php',
            'leo_token' => Tools::getAdminToken('AdminDashboard'),
            'leo_text' => $this->l('This is modules free. You create max 2 item'),
        ));
    }

    public function hookdisplayFooter()
    {
        if (Configuration::get('LEOPOPUPSALE_CONFIG_ACTIVE') == 0) {
            return;
        }

        if (isset($this->context->cookie->cookie_popup)) {
            $popup = Tools::jsonDecode($this->context->cookie->cookie_popup, true);

            $this->smarty->assign(array(
                'leo_time' => $popup['time'],
                'color' => $popup['color'],
                'position' => $popup['position'],
            ));
        }

        return $this->display(__FILE__, 'views/templates/leo-panel.tpl');
    }

    public function hookdisplayFooterAfter($params)
    {
        if (Configuration::get('LEOPOPUPSALE_ACTIVE') != 1) {
            return;
        }

        $id_category = 0;

        if (Configuration::get('LEOPOPUPSALE_CATEGORY') != '') {
            $id_category = (int) Configuration::get('LEOPOPUPSALE_CATEGORY');
        } else {
            $categories = $this->getCategory();

            if (!empty($categories)) {
                $id_category = (int)$categories[0]['id_category'];
            }
        }

        $products = array();
        $customers = array();

        if (Configuration::get('LEOPOPUPSALE_SELECT_TYPE') == 2) {
            $array_orders = Db::getInstance()->executeS(
                'SELECT `id_order`
                FROM `'._DB_PREFIX_.'orders`
                WHERE 1 ORDER BY RAND() LIMIT 10'
            );

            $id_order = '';

            if (!empty($array_orders)) {
                foreach ($array_orders as $key => $array_order) {
                    if ($key == 0) {
                        $id_order .= $array_order['id_order'];
                    } else {
                        $id_order .= ',' . $array_order['id_order'];
                    }
                }
            }

            $products = Db::getInstance()->executeS(
                'SELECT DISTINCT p.`id_product`, pl.`name`, pl.`link_rewrite`
                FROM `'._DB_PREFIX_.'product` p
                LEFT JOIN `'._DB_PREFIX_.'product_lang` pl
                ON p.`id_product` = pl.`id_product`
                LEFT JOIN `'._DB_PREFIX_.'product_shop` pshop
                ON p.`id_product` = pshop.`id_product`
                LEFT JOIN `'._DB_PREFIX_.'category_product` pc
                ON p.`id_product` = pc.`id_product`
                WHERE pc.`id_product`
                IN (SELECT DISTINCT `product_id` FROM `'._DB_PREFIX_.'order_detail` WHERE `id_order` IN('.$id_order.'))
                AND pl.`id_lang` = '.(int)$this->id_lang.'
                AND pshop.`active` = 1'
            );

            $products = $this->getImageProduct($products);

            $customers = Db::getInstance()->executeS(
                'SELECT CONCAT(LEFT(`firstname`, 1), `lastname`) AS `name`, `phone`
                FROM `'._DB_PREFIX_.'address`
                WHERE id_customer
                IN (SELECT DISTINCT `id_customer` FROM `'._DB_PREFIX_.'orders` WHERE `id_order` IN('.$id_order.'))'
            );
        } else if (Configuration::get('LEOPOPUPSALE_SELECT_TYPE') == 3) {
            $products = $this->getProduct(Configuration::get('LEOPOPUPSALE_LIST_PRODUCT'));
            $products = $this->getImageProduct($products);

            $customers = unserialize($this->base64Decode(
                Configuration::get('LEOPOPUPSALE_CUSTOMER')
            ));
        } else {
            $products = $this->getProductByIDCategory($id_category);
        }

        if (!empty($customers)) {
            $customer = $customers[0];
            $customer['phone'] = 'xxx' . substr($customer['phone'], 3);

            foreach ($customers as $key_c => $val_c) {
                $customers[$key_c]['phone'] = 'xxx' . substr($val_c['phone'], 3);
            }
        } else {
            $customer = array(
                'name' => 'John',
                'phone' => 'xxx3456789'
            );

            $customers = array(
                array(
                    'name' => 'John',
                    'phone' => 'xxx3456789',
                ),
                array(
                    'name' => 'Doe',
                    'phone' => 'xxx8888888',
                )
            );
        }

        if (!empty($products)) {
            $product = $products[0];
        }

        $time = 2000;
        $position = 3;
        $color = '#ffffff';

        if (isset($this->context->cookie->cookie_popup)) {
            $popup = Tools::jsonDecode($this->context->cookie->cookie_popup, true);

            $time = $popup['time'];
            $position = $popup['position'];
            $color = $popup['color'];
        } else {
            if (Configuration::get('LEOPOPUPSALE_TIME') != '') {
                $time = Configuration::get('LEOPOPUPSALE_TIME');
            }

            if (Configuration::get('LEOPOPUPSALE_POSITION') != '')
                $position = Configuration::get('LEOPOPUPSALE_POSITION');

            if (Configuration::get('LEOPOPUPSALE_BG') != '')
                $color = Configuration::get('LEOPOPUPSALE_BG');
        }

        $this->smarty->assign(array(
            'theme' => Configuration::get('LEOPOPUPSALE_SELECT_THEME'),
            'color' => $color,
            'type' => Configuration::get('LEOPOPUPSALE_SELECT_TYPE'),
            'time' => $time,
            'products' => $products,
            'product' => $product,
            'position' => $position,
            'customers' => $customers,
            'customer' => $customer,
        ));

        return $this->display(__FILE__, 'views/templates/popup.tpl');
    }

    public function getCategory()
    {
        $sql = 'SELECT c.`id_category`, cl.`name`
        FROM `'._DB_PREFIX_.'category` c
        LEFT JOIN `'._DB_PREFIX_.'category_lang` cl
        ON c.`id_category` = cl.`id_category`
        WHERE c.`id_category`
        IN (SELECT DISTINCT `id_category` FROM `'._DB_PREFIX_.'category_product` WHERE 1)
        AND c.`active` = 1 AND cl.`id_lang` = ' . (int)$this->id_lang;

        return Db::getInstance()->executeS($sql);
    }

    public function getProductByIDCategory($id_category)
    {
        $products = Db::getInstance()->executeS(
            'SELECT p.`id_product`, pl.`name`, pl.`link_rewrite`
            FROM `'._DB_PREFIX_.'product` p
            LEFT JOIN `'._DB_PREFIX_.'product_lang` pl
            ON p.`id_product` = pl.`id_product`
            WHERE p.`id_product`
            IN (SELECT `id_product` FROM `'._DB_PREFIX_.'category_product` WHERE `id_category` = '.(int)$id_category.')
            AND pl.`id_lang` = '.(int)$this->id_lang.'
            LIMIT 10'
        );

        return $this->getImageProduct($products);
    }

    public function getImageProduct($products)
    {
        if (!empty($products)) {
            foreach ($products as $key_p => $product) {
                $products[$key_p]['url'] = $this->context->link->getProductLink($product['id_product']);
                $image = Image::getCover($product['id_product']);

                $products[$key_p]['img'] = $this->context->link->getImageLink(
                    $product['link_rewrite'],
                    $image['id_image'],
                    ImageType::getFormattedName('home')
                );
            }
        }

        return $products;
    }

    public function base64Decode($data)
    {
        return call_user_func('base64_decode', $data);
    }

    public function base64Encode($data)
    {
        return call_user_func('base64_encode', $data);
    }

    public function displayProductSearch($products)
    {
        $this->smarty->assign(array(
            'products' => $products,
        ));

        return $this->fetch('module:leopopupsale/views/templates/leo-product.tpl');
    }
}
